@@ -6,11 +6,15 @@ from __future__ import absolute_import
 
 import gzip
 import json
+import os
+import posixpath
 from copy import deepcopy
 from pathlib import Path, PurePath
-from typing import Generator, Dict, Tuple, Optional
+from typing import Generator, Dict, Tuple
+from urllib.parse import urlparse
 
 from boltons import iterutils
+
 from eodatasets3 import serialise
 
 _DOCUMENT_EXTENSIONS = (".yaml", ".yml", ".json")
@@ -229,68 +233,76 @@ def make_paths_relative(
     for doc_path, value in iterutils.research(
         doc, lambda p, k, v: isinstance(v, PurePath)
     ):
-        value: Path
-
-        if value.is_absolute():
-            if base_directory not in value.parents:
-                if not allow_paths_outside_base:
-                    raise ValueError(
-                        f"Path {value.as_posix()!r} is outside path {base_directory.as_posix()!r} "
-                        f"(allow_paths_outside_base={allow_paths_outside_base})"
-                    )
-                continue
-            value = value.relative_to(base_directory)
-
-        docpath_set(doc, doc_path, str(value))
+        value: PurePath
+        value = relative_path(
+            value, base_directory, allow_paths_outside_base=allow_paths_outside_base
+        )
+        docpath_set(doc, doc_path, value.as_posix())
 
 
-def resolve_absolute_offset(
-    dataset_path: Path, offset: str, target_path: Optional[Path] = None
-) -> str:
+def relative_url(value: str, base: str, allow_paths_outside_base=False) -> str:
     """
-    Expand a filename (offset) relative to the dataset.
+    Make a single url relative to the base url if it is inside it.
 
-    >>> external_metadata_loc = Path('/tmp/target-metadata.yaml')
-    >>> resolve_absolute_offset(
-    ...     Path('/tmp/great_test_dataset'),
-    ...     'band/my_great_band.jpg',
-    ...     external_metadata_loc,
-    ... )
-    '/tmp/great_test_dataset/band/my_great_band.jpg'
-    >>> resolve_absolute_offset(
-    ...     Path('/tmp/great_test_dataset.tar.gz'),
-    ...     'band/my_great_band.jpg',
-    ...     external_metadata_loc,
-    ... )
-    'tar:/tmp/great_test_dataset.tar.gz!band/my_great_band.jpg'
-    >>> resolve_absolute_offset(
-    ...     Path('/tmp/great_test_dataset.tar'),
-    ...     'band/my_great_band.jpg',
-    ... )
-    'tar:/tmp/great_test_dataset.tar!band/my_great_band.jpg'
-    >>> resolve_absolute_offset(
-    ...     Path('/tmp/great_test_dataset.zip'),
-    ...     'band/other/my_great_band.jpg',
-    ... )
-    'zip:/tmp/great_test_dataset.zip!band/other/my_great_band.jpg'
-    >>> resolve_absolute_offset(
-    ...     Path('/tmp/MY_DATASET'),
-    ...     'band/my_great_band.jpg',
-    ...     Path('/tmp/MY_DATASET/ga-metadata.yaml'),
-    ... )
-    'band/my_great_band.jpg'
+    By default, will throw a ValueError if not able to make it relative to the path.
+
+
+    >>> relative_url('file:///g/data/v10/0/2015/blue.jpg', 'file:///g/data/v10/0/2015/odc-metadata.yaml')
+    'blue.jpg'
+    >>> relative_url('https://example.test/2015/images/blue.jpg', 'https://example.test/2015/odc-metadata.yaml')
+    'images/blue.jpg'
+    >>> relative_url('file:///g/data/v10/0/2018/blue.jpg', 'file:///g/data/v10/0/2015/odc-metadata.yaml')
+    Traceback (most recent call last):
+      ...
+    ValueError: Path 'file:///g/data/v10/0/2018/blue.jpg' is outside path 'file:///g/data/v10/0/2015/odc-metadata.yaml'\
+ (allow_paths_outside_base=False)
     """
-    dataset_path = dataset_path.absolute()
 
-    if target_path:
-        # If metadata is stored inside the dataset, keep paths relative.
-        if str(target_path.absolute()).startswith(str(dataset_path)):
-            return offset
-    # Bands are inside a tar file
+    if not value:
+        return value
 
-    if ".tar" in dataset_path.suffixes:
-        return "tar:{}!{}".format(dataset_path, offset)
-    elif ".zip" in dataset_path.suffixes:
-        return "zip:{}!{}".format(dataset_path, offset)
-    else:
-        return str(dataset_path / offset)
+    if not value.startswith(base) and not value.startswith(os.path.dirname(base)):
+        if not allow_paths_outside_base:
+            raise ValueError(
+                f"Path {value!r} is outside path {base!r} "
+                f"(allow_paths_outside_base={allow_paths_outside_base})"
+            )
+        return value
+
+    return _make_relurl(value, base)
+
+
+def _make_relurl(target: str, base: str) -> str:
+    base = urlparse(base)
+    target = urlparse(target)
+    if base.netloc != target.netloc:
+        raise ValueError("target and base netlocs do not match")
+    base_dir = "." + posixpath.dirname(base.path)
+    target = "." + target.path
+    return posixpath.relpath(target, start=base_dir)
+
+
+def relative_path(
+    value: PurePath, base_directory: PurePath, allow_paths_outside_base=False
+) -> PurePath:
+    """
+    Make a single path relative to the base directory if it is inside it.
+
+    By default, will throw a ValueError if not able to make it relative to the path.
+
+    >>> val =  PurePath('/tmp/minimal-pkg/loch_ness_sightings_2019-07-04_blue.tif')
+    >>> base = PurePath('/tmp/minimal-pkg')
+    >>> relative_path(val, base).as_posix()
+    'loch_ness_sightings_2019-07-04_blue.tif'
+    """
+    if not value or not value.is_absolute():
+        return value
+
+    if base_directory not in value.parents:
+        if not allow_paths_outside_base:
+            raise ValueError(
+                f"Path {value.as_posix()!r} is outside path {base_directory.as_posix()!r} "
+                f"(allow_paths_outside_base={allow_paths_outside_base})"
+            )
+        return value
+    return value.relative_to(base_directory)
