@@ -15,6 +15,8 @@ from click import secho
 from eodatasets3 import wagl
 from eodatasets3.ui import PathPath
 
+DEFAULT_MATURITY = wagl.ProductMaturity.stable
+
 
 @click.command(help=__doc__)
 @click.option(
@@ -45,6 +47,13 @@ from eodatasets3.ui import PathPath
     is_flag=True,
     default=True,
 )
+@click.option(
+    "--product-maturity",
+    type=click.Choice(wagl.ProductMaturity.__members__, case_sensitive=False),
+    help=f"Product maturity status (default: {DEFAULT_MATURITY.name})",
+    default=DEFAULT_MATURITY.name,
+    callback=lambda c, p, v: wagl.ProductMaturity[v.lower()] if v else None,
+)
 @click.option(
     "--with-oa/--no-oa",
     "with_oa",
@@ -73,6 +82,7 @@ def run(
     h5_file: Path,
     products: Sequence[str],
     with_oa: bool,
+    product_maturity: wagl.ProductMaturity,
     allow_missing_provenance: bool,
     oa_resolution: Optional[float],
 ):
@@ -101,6 +111,7 @@ def run(
                 dataset_id, dataset_path = wagl.package(
                     out_directory=output,
                     granule=granule,
+                    product_maturity=product_maturity,
                     included_products=products,
                     include_oa=with_oa,
                     oa_resolution=oa_resolution,
